#ifndef GPMI_OS_DEP_H
#define GPMI_OS_DEP_H

#ifdef WIN32
char *basename(char *path);
char *dirname(char *path);
#endif

#ifdef DLCOMPAT
#	include <dl-compat.h>
#else
#	include <dlfcn.h>
#endif
#include <limits.h>

/* Define a library handle */
#define gpmi_library_handle void *

#ifndef PATH_MAX
#ifdef MAX_PATH
#define PATH_MAX MAX_PATH
#else
#define PATH_MAX 256
#endif
#endif

#ifdef WIN32
#ifdef __CYGWIN__
#define PRINTF_PTR "%p"
#define PRINTF_PTR_CAST
#else
#define PRINTF_PTR "0x%x"
#define PRINTF_PTR_CAST (size_t)
#endif
#define GPMI_PATH_SEP_STR ";"
#define GPMI_PATH_SEP_CHAR ';'
#else
#define PRINTF_PTR "%p"
#define PRINTF_PTR_CAST
#define GPMI_PATH_SEP_STR ":"
#define GPMI_PATH_SEP_CHAR ':'
#endif

#ifdef UNIX
#include <libgen.h>
#endif

#ifndef OS_DEP_MACROS_ONLY

/* Functions for the user */
int  gpmi_install_root(char *inst_root, int inst_root_size);

typedef enum gpmi_env_e {
	gpmi_env_home,
} gpmi_env_t;

/* Low level functions */
void  gpmi_library_add_path(const char *dir, const char *library_name, char *lib_file_name, char *dep_file_name, int path_size);
int   gpmi_library_load_lowlevel(gpmi_library *library, char *file_name);
void  gpmi_library_unload_lowlevel(gpmi_library *library);
void *gpmi_dlsym_lowlevel(gpmi_library *library, const char *name);
void *gpmi_dlopen_global();
char *gpmi_getenv(gpmi_env_t env);


/* Path assembly */
#ifdef WIN32
	char *gpmi_init_path(char *path);
#else
	/* Keep fast way for non-win32 */
	#define gpmi_init_path(path) strdup(GPMI_INST_ROOT path);
#endif

#else
/* Make header-file free to readd */
#	undef GPMI_OS_DEP_H
#endif

#endif /* GPMI_OS_DEP_H */
