#include "cli.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "qparse.h"
#include "gpmi/debug.h"

/**
 * Returns the extension of this script-module.
 */
char *Cli_GetExtension()
{
	return strdup("");
}

/**
 * Reloads the script on the fly.
 */
int Cli_Rehash(void *client_data)
{
	/* TODO */
	return 0;
}

/**
 * Loads a script.
 */
int Cli_LoadScript(void *client_data, const char *filename)
{
	Cli_ClientData *mydata = client_data;
	gpmi_module *mod;
	FILE *f;
	char buff[512];
	char *s, *dir, *before, *after;
	int res;

	/* We need examine the shebang to see how to interpret the file. We
	cut down the first word and check the part after the last '/'; in other
	words, the script must start with something like '#!/bin/bash' where
	the shebang ('#!'), the path ('/bin/') is cut down. Options are
	allowed, but ignored (e.g. #!/bin/awk -Winteractive). */

	/* Open the file */
	f = fopen(filename, "r");
	if (f == NULL)
		return -1;

	/* Read first line */
	*buff = '\0';
	fgets(buff, sizeof(buff), f);
	fclose(f);

	/* Check for shebang */
	if ((buff[0] != '#') || (buff[1] != '!'))
		return -2;

	/* Cut at first whitespace */
	s = strpbrk(buff + 2, " \t\r\n");
	if (s != NULL)
		*s = '\0';

	/* set s to the last '/' or to the last char of the shebang */
	s = strrchr(buff + 2, '/');
	if (s == NULL)
		s = buff + 1;

	/* Increase s so we have the first char of ther actual interperter name */
	s++;

	mod = gpmi_get_current_module();

	/* Read the config to find the scripts we need to combine */
	dir = before = after = NULL;
	if (mod->path != NULL)
		res = cli_read_confs(mod->path, s, &dir, &before, &after);
	else
		res = cli_read_confs_multi(GPMI_PATH_MODULES, s, &dir, &before, &after);

	cli_dprintf("dir=%s\n", dir);
	if ((res != 0) || (dir == NULL) || (before == NULL) || (after == NULL))
		return -3;

	/* Fork the interpreter */
	mydata->c = cli_fork(dir, before, filename, after);
	if (mydata->c == NULL)
		return -4;

	/* Parse the output of the MAIN part of the script */
//	cli_send_string(mydata->c, "NOP\n");
	cli_parse(mydata->c, mydata->read_timeout);

	free(before);
	free(after);
	free(dir);

	return 0;
}

/**
 * Execute a dynamic command.
 **/
int Cli_DynamicPackageCommand(char *command_string)
{
	Cli_ClientData *mydata = GPMI_Script_GetClientData();
	char *(*proc)(int argc, char *argv[]);
	char **argv;
	char *s;
	int argc;

	argc = qparse(command_string, &argv);

	if (strcmp(argv[0], "cli_print") == 0) {
		printf("%s\n", command_string+10);
		fflush(stdout);
		goto end;
	}
	if (strcmp(argv[0], "cli_dprint") == 0) {
		cli_dprintf("%s\n", command_string+11);
		goto end;
	}

	proc = GPMI_Script_FindProc(argv[0]);
	s = proc(argc, argv);

	cli_dprintf( "[CLI] dynres: '%s'\n", s);
	cli_send_string_now(mydata->c, "Result ");
	if (s != NULL) {
		cli_send_string_now(mydata->c, s);
		free(s);
	}
	cli_send_string_now(mydata->c, "\n");

	end:;
	/*XXX: leak*/
//	qparse_free(argc, &argv);
	return 1;
}

/**
 * Register a command in our command hash
 */
void Cli_RegisterCommand(void *client_data, const char *name)
{
	/* Nothing to do here */
}

/**
 * Call a function of the script by name, passing params.
 */
void Cli_RunFunction(void *client_data, const char *function, int argc, const char **argv)
{
	Cli_ClientData *mydata = client_data;
	int n, l;
	char *buff;
	int allocated, used;

	/* Allocate buff, place function in it and add a space */
	l = strlen(function);
	allocated = l + 512;
	buff = malloc(allocated);
	strcpy(buff, "Event ");
	used = 6;
	memcpy(buff + used, function, l);
	used += l;
	buff[used] = ' ';
	used++;

	/* Add all the argvs and extend the buffer as needed */
	for(n = 0; n < argc; n++) {
		l = strlen(argv[n]);
		if (l + used + 1 >= allocated) {
			allocated += l + used + 512;
			buff = realloc(buff, allocated);
		}
		memcpy(buff + used, argv[n], l);
		used += l;
		buff[used] = ' ';
		used++;
	}
	buff[used] = '\n';
	used++;
	buff[used] = '\0';

	cli_send_string_now(mydata->c, buff);

	if (mydata->c->state != cli_st_waitfin)
		cli_parse(mydata->c, mydata->read_timeout);

	free(buff);
}

int Cli_Initialize(void **client_data)
{
	Cli_ClientData *MyData;

	MyData = malloc(sizeof(Cli_ClientData));
	memset(MyData, 0, sizeof(Cli_ClientData));

	*client_data = MyData;
	MyData->c    = NULL;
	MyData->read_timeout = 1000;

	return 0;
}

void Cli_Destroy(void **client_data)
{
	Cli_ClientData *mydata = *client_data;

	/* XXX: free mydata fields! */
	cli_send_string(mydata->c, "Quit");
	cli_parse(mydata->c, 10);
	/* Finally free up the client-data */
	free(mydata);
}
